unit Command;

{$IMPORTEDDATA OFF}

interface

uses NTypes, NLib;

type
  ECommandError = class(EError);

var
  Suspended: Boolean;
  MenuItems: array of TWideStringArray;

procedure DispatchCommand(const Args: array of WideString);

implementation

uses Windows, NSys, NForm, NConf, NExpand, NDesktop, WinMan, Main, NotifyIcon;

function SIntDef(const S: string): Integer;
begin
  try
    Result := SInt(S);
  except
    on EValueError do Result := 0;
  end;
end;

procedure exec(const Args: array of WideString);
var
  a: TWideStringArray;
  dir: WideString;
  opt: WideChar;
begin
  dir := '';
  with ArgParser(Args) do begin
    while GetOpt(opt) do
      case opt of
        'd': dir := GetArg;
      end;
    a := Args;
  end;
  if Length(a) = 0 then
    raise ECommandError.Create('filename not specified');
  try
    Open(a, dir);
  except
    on E: EOSError do raise ECommandError.Create(a[0]+': '+E.Msg);
  end;
end;

procedure menu(const Args: array of WideString);
var
  f: TFile;
  fwnd: HWND;
  i: Integer;
  line: string;
  pt: TPoint;
  rc: TRect;
begin
  if Length(Args) = 0 then
    raise ECommandError.Create('filename not specified');

  with TPopupMenu.Create do
    try
      SetLength(MenuItems, 1024);
      try
        f := TFile.Open(JoinPath([ProfDir, Args[0]]));
        try
          i := 0;
          while TConf.ReadLine(f, line) do begin
            if i = Length(MenuItems) then
              SetLength(MenuItems, i + i div 4);
            MenuItems[i] := SplitCommandLineW(DecodeUTF8(line));
            if MenuItems[i][0] = '--' then
              BreakItem
            else
              AppendItem(IDM_MENUITEM_START + i, MenuItems[i][0]);
            Inc(i);
          end;
        finally
          f.Free;
        end;
      except
        on E: EOSError do raise ECommandError.Create(E.Path + ': ' + E.Msg);
      end;

      // to hold mouse cursor position
      GetCursorPos(pt);
      SetWindowPos(MainWnd, 0, pt.x, pt.y, 0, 0, SWP_NOSIZE or SWP_NOZORDER);
      fwnd := GetForegroundWindow;
      SetForegroundWindow(MainWnd);
      rc := GetWorkArea;
      // top of "centering" is 45% from MSDN window management guidelines
      PopupCenter(MainWnd, (rc.Right-rc.Left) div 2, Trunc((rc.Bottom-rc.Top)*0.45));
      if GetForegroundWindow = MainWnd then
        SetForegroundWindow(fwnd);
    finally
      Free;
    end;
end;

procedure suspend(const Args: array of WideString);
begin
  Suspended := not Suspended;
  if Suspended then begin
    SuspendHotKeys;
    SetNotifyIconState(True);
  end else begin
    UnsuspendHotKeys;
    SetNotifyIconState(False);
  end;
end;

procedure window(const Args: array of WideString);
var
  l, t, r, b, w, h: Integer;
  rev, center: Boolean;
  opt: WideChar;
  position: WideString;
begin
  if Length(Args) = 0 then
    raise ECommandError.Create('command not specified');

  if Args[0] = 'foreground' then
  begin
    rev := False;
    with ArgParser(SliceArrayW(Args, 1)) do begin
      while GetOpt(opt) do
        case opt of
          'r': rev := True;
        end;
      Foreground(Args[0], rev);
    end;
  end
  else if Args[0] = 'info' then
    with GetInfo do begin
      if ErrorPosition then
        position := 'Cannot get position'
      else
        position := Format('left={} top={} width={} height={}',
          [Left, Top, Width, Height]);
      MsgBox(Format(
          'Handle:'#9#9'{}'#10+
          'Caption:'#9#9'{}'#10+
          'Class Name:'#9'{}'#10+
          'Position:'#9#9'{}',
          [Handle, Caption, ClassName, Position]),
          'window info - ' + AppName);
    end
  else if Args[0] = 'minimize' then
    Minimize
  else if Args[0] = 'minimizeall' then
    MinimizeAll
  else if Args[0] = 'pos' then begin
    t := spUndef;
    r := spUndef;
    b := spUndef;
    l := spUndef;
    w := spUndef;
    h := spUndef;
    center := False;
    with ArgParser(SliceArrayW(Args, 1)) do
      while GetOpt(opt) do
        case opt of
          'c': center := True;
          't': begin
                 t := SIntDef(GetArg);
                 b := spUndef;
               end;
          'r': begin
                 r := SIntDef(GetArg);
                 l := spUndef;
                end;
          'b': begin
                 b := SIntDef(GetArg);
                 t := spUndef;
               end;
          'l': begin
                 l := SIntDef(GetArg);
                 r := spUndef;
                end;
          'w': w := SIntDef(GetArg);
          'h': h := SIntDef(GetArg);
        end;
    SetPosition(l, t, r, b, w, h, center);
  end else if Args[0] = 'togglemaximize' then
    ToggleMaximize
  else
    raise ECommandError.Create('unknown command ''' + Args[0] + '''');
end;

function GetValue(const S: WideString): TWideStringArray;
var
  rc: TRect;
begin
  SetLength(Result, 1);
  if S[1] = '$' then begin
    if S = '$_foregroundwindow' then
      Result[0] := IStr(GetForegroundWindow)
    else if S = '$_workareaheight' then begin
      rc := GetWorkArea;
      Result[0] := IStr(rc.Bottom - rc.Top);
    end else if S = '$_workareawidth' then begin
      rc := GetWorkArea;
      Result[0] := IStr(rc.Right - rc.Left);
    end else
      Result[0] := GetEnv(Copy(S, 2, Length(S)-1));
  end else
    Result[0] := S;
end;

procedure DispatchCommand(const Args: array of WideString);
type
  TCommandProc = procedure (const Args: array of WideString);
  TCommand = record
    Name: WideString;
    Proc: TCommandProc;
  end;
const
  Commands: array [0..3] of TCommand = (
    (Name: 'exec'; Proc: exec),
    (Name: 'menu'; Proc: menu),
    (Name: 'suspend'; Proc: suspend),
    (Name: 'window'; Proc: window)
    );
var
  i: Integer;
  cmdproc: TCommandProc;
  expandedargs: TWideStringArray;
begin
  expandedargs := Expand(Args, GetValue);
  cmdproc := nil;
  for i := 0 to High(Commands) do
    if Commands[i].Name = expandedargs[0] then begin
      cmdproc := Commands[i].Proc;
      Break;
    end;
  if not Assigned(cmdproc) then begin
    MsgBox('unknown command '''+expandedargs[0]+'''');
    Exit;
  end;
  try
    cmdproc(SliceArrayW(expandedargs, 1));
  except
    on E: ECommandError do MsgBox(expandedargs[0]+': '+E.Msg);
  end
end;

end.
